#include "Users.h"


Users::Users(Persistence &p): persistence(p){
}
Users::~Users()
{
}
bool Users::ImportFromPersistence(){
    Serial.println("Importing Users from Persistence");
    DynamicJsonDocument user_persistence = persistence.readUsers();
    JsonArray array = user_persistence.as<JsonArray>();
    for(JsonObject userdata : array){
        User *imported = new User();
        imported->uid= userdata["ID"].as<unsigned long>();
        if(_userdb.count(*imported))
            continue;
        imported->first_name = userdata["FIRST_NAME"].as<String>();
        imported->last_name = userdata["LAST_NAME"].as<String>();
        imported->rfid_uid = userdata["RFID_UID"].as<String>();
        imported->user_pin = userdata["USER_PIN"].as<String>();
        imported->enabled = userdata["ENABLED"].as<bool>();
        this->_userdb.insert(*imported);
    }
    Serial.println("User import is done!");
    return true;
}

bool Users::ExportToPersistence(){
    Serial.println("Exporting Users to Persistence");
    size_t user_amount = _userdb.size()+2;
    Serial.print( " User Amount: " );
    Serial.print(user_amount);
    size_t capacity = JSON_ARRAY_SIZE(user_amount) + user_amount*JSON_OBJECT_SIZE(6);
    Serial.print(" JSON Capacity");
    Serial.print(capacity);
     Serial.print(" Free Heap: ");
     Serial.print(ESP.getFreeHeap());
    //DynamicJsonDocument<CAPACITY> doc;
    DynamicJsonDocument doc(capacity);

    for(Users::User u : _userdb){
        JsonObject exported_user = doc.createNestedObject();
        exported_user["ID"] = u.uid;
        exported_user["FIRST_NAME"] = u.first_name;
        exported_user["LAST_NAME"] = u.last_name;
        exported_user["RFID_UID"] = u.rfid_uid;
        exported_user["USER_PIN"] = u.user_pin;
        exported_user["ENABLED"] = u.enabled;
    }
    persistence.saveUsers(doc);
    Serial.println("User export is done!");
    return true;
}
bool Users::checkPin(String pin_code, std::vector<Users::User> *logon_users)
{
    std::copy_if(this->_userdb.begin(), this->_userdb.end(), inserter(*logon_users, logon_users->end()), [=](user_account user)
                 { return pin_code == user.user_pin&&user.enabled==true; });
    return (logon_users->size() > 0);
}

bool Users::checkRfid(String rfid_code, std::vector<Users::User> *logon_users)
{
    std::copy_if(this->_userdb.begin(), this->_userdb.end(), inserter(*logon_users, logon_users->end()), [=](user_account user)
                 { return rfid_code == user.rfid_uid&&user.enabled==true; });
    return (logon_users->size() > 0);
}

unsigned long Users::addUser(String first_name, String last_name, String rifd_uid, String user_pin)
{
    Serial.println("Adding User:" + first_name);
    unsigned long new_id = this->_userdb.rbegin()->uid + 1;
    this->_userdb.insert(user_account{.uid = new_id, .first_name = first_name, .last_name = last_name, .rfid_uid = rifd_uid, .user_pin = user_pin});
    return new_id;
}

bool Users::delUser(unsigned long id)
{
    return this->_userdb.erase(user_account{.uid = id})>0;
}

bool Users::updateUser(unsigned long id, ATTRIBUTES attr, String value)
{
    auto  modify = this->_userdb.find(user_account{.uid = id});
    switch (attr)
    {
    case FIRST_NAME:
        modify->first_name = value;
        break;
    case LAST_NAME:
        modify->last_name = value;
        break;
    case RFID_UID:
        modify->rfid_uid = value;
        break;
    case USER_PIN:
        modify->user_pin = value;
        break;

    default:
        break;
    }
    return false;
}

size_t Users::countUsers(){
    return _userdb.size();
}
String Users::toString(User *user)
{
    return "UID: " + String(user->uid) + " Name: " + user->first_name + " " + user->last_name;
}
void Users::PrintAllToSerial(){
    Serial.println("#All Users:");
    for(Users::User u : _userdb){
        Serial.println(toString(&u));
    }
}